﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Threading.Tasks;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.ComponentModel;

namespace MyGaugeClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        int recv;
        byte[] data = new byte[1024];
        int port = 8765;
        string localHost = "127.0.0.1";
        IPAddress localAddr = null;
        TcpListener newsock = null;
        TcpClient server = null;
        Boolean connected = false;

        BackgroundWorker backgroundWorker = null;
        int sliderValue = 0;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void connectButton_Click(object sender, RoutedEventArgs e)
        {
            // Start the background worker
            backgroundWorker.RunWorkerAsync();
        }

        private void disconnectButton_Click(object sender, RoutedEventArgs e)
        {
            connected = false;
            mySlider.Value = 0;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            backgroundWorker = new BackgroundWorker();

            // To report progress from the background worker we need to set this property
            backgroundWorker.WorkerReportsProgress = true;

            // This event will be raised on the worker thread when the worker starts
            backgroundWorker.DoWork += new DoWorkEventHandler(backgroundWorker_DoWork);

            // This event will be raised when we call ReportProgress
            backgroundWorker.ProgressChanged += new ProgressChangedEventHandler(backgroundWorker_ProgressChanged);
        }

        // On worker thread so do our thing!
        void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            // Your background task goes here
            try
            {
                server = new TcpClient(localHost, port);
                connected = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                //Console.WriteLine("Unable to connect to server");
                return;
            }
            //Console.WriteLine("TcpClient server constructed");

            NetworkStream ns = server.GetStream();
            //Console.WriteLine("NetworkStream ns assigned {0}", ns.ToString());

            int recv = ns.Read(data, 0, data.Length);
            //Console.WriteLine("Received {0} bytes", recv);

            string stringData = Encoding.ASCII.GetString(data, 0, recv);
            //Console.WriteLine("Received: " + stringData);

            while (connected)
            {
                string input = sliderValue.ToString(); 

                //if (input == "exit")
                //    break;

                ns.Write(Encoding.ASCII.GetBytes(input), 0, input.Length);
                ns.Flush();

                data = new byte[1024];
                recv = ns.Read(data, 0, data.Length);
                stringData = Encoding.ASCII.GetString(data, 0, recv);
                //Console.WriteLine(stringData);

                backgroundWorker.ReportProgress(int.Parse(stringData));
            }

            //Console.WriteLine("Disconnecting from server...");

            ns.Close();
            server.Close();

            backgroundWorker.ReportProgress(0);
        }

        // Back on the 'UI' thread so we can update the progress bar
        void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // The progress percentage is a property of e
            myGauge.CurrentValue = e.ProgressPercentage;
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {

        }

        private void mySlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            sliderValue = (int)mySlider.Value;
        }
    }
}
