﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.Threading.Tasks;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.ComponentModel;

namespace MyGaugeServer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        int recv;
        byte[] data = new byte[1024];
        int port = 8765;
        string localHost = "127.0.0.1";
        IPAddress localAddr = null;
        TcpListener newsock = null;
        BackgroundWorker backgroundWorker = null;
        bool allWorkDone = false;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Closed(object sender, EventArgs e)
        {

        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            localAddr = IPAddress.Parse(localHost);

            backgroundWorker = new BackgroundWorker();

            // To report progress from the background worker we need to set this property
            backgroundWorker.WorkerReportsProgress = true;

            // This event will be raised on the worker thread when the worker starts
            backgroundWorker.DoWork += new DoWorkEventHandler(backgroundWorker_DoWork);

            // This event will be raised when we call ReportProgress
            backgroundWorker.ProgressChanged += new ProgressChangedEventHandler(backgroundWorker_ProgressChanged);

            backgroundWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);
        }

        private void startServerButton_Click(object sender, RoutedEventArgs e)
        {
            // Start the background worker
            backgroundWorker.RunWorkerAsync();

            startServerButton.IsEnabled = false;
            stopServerButton.IsEnabled = true;
        }

        private void stopServerButton_Click(object sender, RoutedEventArgs e)
        {
            allWorkDone = true;
            stopServerButton.IsEnabled = false;
        }

        // On worker thread so do our thing!
        void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            while (!allWorkDone)
            {
                // Your background task goes here
                // TcpListener server = new TcpListener(port);
                newsock = new TcpListener(localAddr, port);
                newsock.Start();
                //Console.WriteLine("TcpListener started, Waiting for a client...");
                //MessageBox.Show("Waiting for client");

                TcpClient client = newsock.AcceptTcpClient();
                NetworkStream ns = client.GetStream();

                //MessageBox.Show("client accepted");

                string welcome = "Welcome to my test server";
                data = Encoding.ASCII.GetBytes(welcome);
                ns.Write(data, 0, data.Length);
                //MessageBox.Show("Initial server message sent");

                while (true)
                {
                    data = new byte[1024];
                    recv = ns.Read(data, 0, data.Length);
                    if (recv == 0)
                        break;

                    // Change the gauge value here using reportProgress
                    int value = int.Parse(Encoding.ASCII.GetString(data, 0, recv));

                    backgroundWorker.ReportProgress(value);
                    //Console.WriteLine(Encoding.ASCII.GetString(data, 0, recv));
                    ns.Write(data, 0, recv);
                }

                ns.Close();
                client.Close();
                newsock.Stop();

                backgroundWorker.ReportProgress(0);

                if (allWorkDone)
                    break;
            }

            startServerButton.IsEnabled = true;
        }
        // Back on the 'UI' thread so we can update the progress bar
        void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // The progress percentage is a property of e
            myGauge.CurrentValue = e.ProgressPercentage;
        }

        void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            stopServerButton.IsEnabled = false;
            startServerButton.IsEnabled = true;
        }

        
    }
}
